package cz.euweb.icebearsoft.saxon;

// For linkage to Saxon
import net.sf.saxon.expr.*;
import net.sf.saxon.om.*;
import net.sf.saxon.trans.DynamicError;
import net.sf.saxon.trans.XPathException;

// For finding and formatting the file modificaton time
import java.io.File;
import java.util.Date;
import java.util.Calendar;
import java.text.SimpleDateFormat;
import java.text.DecimalFormat;
import java.net.URLDecoder;
import java.io.UnsupportedEncodingException;

// For debugging from the command line
import java.util.TimeZone;


/**
This file implements an extension utility for Saxon. It was tested with version 8.5
@author Zdenek Wagner, http://icebearsoft.euweb.cz
*/
public class ExtUtils {

/**
The class is never instantiated.
*/
private ExtUtils() {}

/**
This function implements the <code>last-modified()</code> function without any argument. It returns
the modification time of the file containing the context node.
@param c the context node supplied by Saxon
@return file modification time formatted according to ISO 8601 or an empty string if the context
node is not present in a local file
@exception XPathException XPath dynamic error reported back to Saxon
*/
public static String lastModified(XPathContext c) throws XPathException {
Item item = c.getContextItem();
if (item == null) {
  DynamicError e = new DynamicError("The context item for lastModified() is not set");
  e.setXPathContext(c);
  throw e;
}
if (item instanceof NodeInfo) {
  return lastModified((NodeInfo)item);
} else {
  return "";
}
}

/**
This function implements the <code>last-modified(node)</code> function with one argument which
must be a node. It returns the modification time of the file containing the context node.
@param node the node supplied by a user
@return file modification time formatted according to ISO 8601 or an empty string if the supplied
node is not present in a local file
@exception XPathException XPath dynamic error reported back to Saxon
*/
public static String lastModified(NodeInfo node) {
String sysid = node.getSystemId();
if (sysid.startsWith("file:")) {
  String fn;
  // It's hopefully the filesystem encoding
  String enc = System.getProperty("file.encoding", "UTF-8");
  try {
    fn = URLDecoder.decode(sysid, enc);
  } catch (UnsupportedEncodingException e) {
    try {
      fn = URLDecoder.decode(sysid, "UTF-8"); // If the property is empty...
    } catch (UnsupportedEncodingException ee) {
      return ""; // What to do if the encoding is not supported?
    }
  }
  return fileTimestamp(fn.substring(5));
} else {
  return "";
}
}

/**
This function determines the file modification time. It can be called from the stylesheet as file-timestamp(fn).
@param fn file name, type xs:string
@return file modification time formatted according to ISO 8601 or an empty string if the file is
not found
*/
public static String fileTimestamp(String fn) {
File f = new File(fn);
long flastmod = f.lastModified();
if (flastmod == 0) return "";
Calendar c = Calendar.getInstance();
c.setTimeInMillis(flastmod);
Date d = c.getTime();
// Format the date/time according to ISO 8601
SimpleDateFormat fmt1 = new SimpleDateFormat("yyyy-MM-dd");
SimpleDateFormat fmt2 = new SimpleDateFormat("HH:mm:ss");
int tz = (c.get(Calendar.ZONE_OFFSET) + c.get(Calendar.DST_OFFSET)) / 60 / 1000;
int tzhours = Math.abs(tz) / 60;
int tzminutes = Math.abs(tz) % 60;
DecimalFormat dfmt = new DecimalFormat("00");
String tzstring = dfmt.format(tzhours) + ":" + dfmt.format(tzminutes);
return fmt1.format(d) + "T" + fmt2.format(d) +
  (tz == 0 ? "Z" : tz < 0 ? "-" + tzstring : "+" + tzstring);
}

/**
Command line debugging and testing function. It displays the system properties, the name of the
time zone and the file modification times.
@param argv list of file names (may be relative)
*/
public static void main (String argv[]) {
Calendar c = Calendar.getInstance();
System.getProperties().list(System.out);
System.out.println("Time zone: " + c.getTimeZone().getDisplayName());
int i;
for (i = 0;  i < argv.length;  i++) {
  System.out.println();
  System.out.println("File name: " + argv[i]);
  System.out.println("Last modified: " + fileTimestamp(argv[i]));
}
}

}
